c  HoloInvert.f
c  version 2.0b
c
c  this program reconstructs atomic images from holographic intensities.
c
c  please acknowledge use of this program.  please contact:
c  p. m. len
c  physics department
c  university of california, davis
c  davis, ca  95616
c
c  s. thevuthasan, may 93     	original program
c  a. p. kaduwela, jan 95     	reoptimized for scalar machines
c  p. m. len, mar 96		final modifications      
c  p. m. len, jul 96		implementation of small cone routine
c  p. m. len, sep 96            final modifications
c
c  to compile this code on a cray:
c  cf77 -Zv HoloInvert.f -o HoloInvert
c
c  to compile this code on a SUN:
c  f77 -r8 -i4 -O3 HoloInvert.f -o HoloInvert
c
c  initialization and parameterization
	real k(40), x(256), y(256), z(256)
	real k0, krkr, theta(18,72), phi(18,72), xK(40,18,72)
        real stheta(18,72), ctheta(18,72)
	real kx(18,72), ky(18,72), kz(18,72)
	complex kern, dUr, Ur
	integer nx, nz, nk, ntheta, nphi
	character*80 FilexK, FileUr
	parameter(pi = 3.141592654)
	parameter(degtorad = pi/180.0)
c
c  read in reconstruction integration parameters
	open(unit = 10, file = 'HoloInvert.in')
	  read(10,10) FilexK
	  read(10,*) ntheta
	  read(10,*) nphi	
cccc	  read(10,*) k0
cccc	  read(10,*) dk
	  read(10,*) nk
	  read(10,*) rotate
	  read(10,10) FileUr
	  read(10,*) xmin, xmax, dx
	  read(10,*) ymin, ymax, dy	
	  read(10,*) zmin, zmax, dz	
	close(10)
	nx = ((xmax - xmin)/dx) + 1
	ny = ((ymax - ymin)/dy) + 1 
	nz = ((zmax - zmin)/dz) + 1 
c	k0 = 0.5121365847*sqrt(energy)
10	format(a80)
c
	call read_window(coswindow)
c
c  initialization of k, x, y, and z arrays
	do 60 ix = 1, nx
	  x(ix) = xmin + (ix-1)*dx
60	continue
	do 70 iy = 1, ny
	  y(iy) = ymin + (iy-1)*dy
70	continue
	do 80 iz = 1, nz
	  z(iz) = zmin + (iz-1)*dz
80	continue
cccc	do 90 ik = 1, nk
cccc	  k(ik) = k0 + (ik-1)*dk
cccc90	continue
c
c  main read-in routine
	open(unit = 20, file = FilexK)
	do 100 ik = 1, nk
	  do 101 itheta = 1, ntheta
	    do 102 iphi = 1, nphi
              read(20,*) theta(itheta,iphi), phi(itheta,iphi),
     &                   xK(ik,itheta,iphi)
	      theta(itheta,iphi) = theta(itheta,iphi)*degtorad
	      phi(itheta,iphi) = (phi(itheta,iphi) - rotate)*degtorad
102         continue
101	  continue
100	continue
	close(20)
c
c  initialization of angles and trigonometrics
        do 200 itheta = 1, ntheta
          do 200 iphi = 1, nphi
	    ctheta(itheta,iphi) = cos(theta(itheta,iphi))
	    stheta(itheta,iphi) = sin(theta(itheta,iphi))
	    kx(itheta,iphi) = ctheta(itheta,iphi)*cos(phi(itheta,iphi))
	    ky(itheta,iphi) = ctheta(itheta,iphi)*sin(phi(itheta,iphi))
	    kz(itheta,iphi) = stheta(itheta,iphi)
200	continue
c
c  loop over each position in U(r) 
	open(unit = 30, file = FileUr)
	do 300 ix = 1, nx
	  do 301 iy = 1, ny
	    do 302 iz = 1, nz
	    dUr = (0., 0.)
	    r = sqrt(x(ix)*x(ix) + y(iy)*y(iy) + z(iz)*z(iz))
c  integrate over hologram solid angle
	    do 303 itheta = 1, ntheta
	      do 304 iphi = 1, nphi
		xkx = x(ix)*kx(itheta,iphi)
		yky = y(iy)*ky(itheta,iphi)
	    	zkz = z(iz)*kz(itheta,iphi)
		costhetark = abs((xkx + yky + zkz)/r)
	        if (costhetark .gt. coswindow) then
c  phase-sum over different wavenumbers
		  do 305 ik = 1, nk
                    krkr = k(ik)*(xkx + yky + zkz) - k(ik)*r
                    kern = cmplx(cos(krkr),sin(krkr))
	dUr = dUr + xK(ik,itheta,iphi)*kern*k(ik)*k(ik)*ctheta(itheta,iphi)
305	          continue
		endif
c  end phase-sum
304	        continue
303	      continue
	      Ur = dUr
cccc	      write(30,30) Ur
	      write(30,30) x(ix), y(iy), z(iz), Ur
c  end hologram solid angle integration
302	    continue
301	  continue
300     continue
c  end loop over each position in U(r)
cccc30	format(2(2x,e12.5))
30      format(3(2x,f6.2),2(2x,e12.5))
c
	close(30)
	end
c
        subroutine read_window(coswindow)
c  this subroutine reads in the small-cone half-angle 
c
        real window, coswindow
c
        open(unit = 40, file = 'window.in')
          read(40,*) window
        close(40)
	coswindow = cos(window*3.141592654/180.)
        return
        end
