c  HoloInvert.f
c  version 2.0d
c
c  this program reconstructs atomic images from holographic intensities.
c
c  please acknowledge use of this program.  please contact:
c  p. m. len
c  physics department
c  university of california, davis
c  davis, ca  95616
c
c  s. thevuthasan, may 93     	original program
c  a. p. kaduwela, jan 95     	reoptimized for scalar machines
c  p. m. len, mar 96		final modifications      
c  p. m. len, jul 96		implementation of Hofmann-Schindler
c  p. m. len, sep 96            final modifications
c
c  to compile this code on a cray:
c  cf77 -Zv HoloInvert.f -o HoloInvert
c
c  to compile this code on a SUN:
c  f77 -r8 -i4 -O3 HoloInvert.f -o HoloInvert
c
c  initialization and parameterization
	real k(40), x(256), y(256), z(256)
	real k0, krkr, theta(18,72), phi(18,72), xK(40,18,72)
        real stheta(18,72), ctheta(18,72)
	real kx(18,72), ky(18,72), kz(18,72)
	complex kern, dUr
	real Ur_ave, Ur(256,256,256)
	integer nx, nz, nk, ntheta, nphi
	character*80 FilexK, FileUr
	parameter(pi = 3.141592654)
	parameter(degtorad = pi/180.0)
c
	real fk(40,0:180)
c
c  read in reconstruction integration parameters
	open(unit = 10, file = 'HoloInvert.in')
	  read(10,10) FilexK
	  read(10,*) ntheta
	  read(10,*) nphi	
	  read(10,*) k0
	  read(10,*) dk
	  read(10,*) nk
	  read(10,*) rotate
	  read(10,10) FileUr
	  read(10,*) xmin, xmax, dx
	  read(10,*) ymin, ymax, dy	
	  read(10,*) zmin, zmax, dz	
	close(10)
	nx = ((xmax - xmin)/dx) + 1
	ny = ((ymax - ymin)/dy) + 1 
	nz = ((zmax - zmin)/dz) + 1 
c	k0 = 0.5121365847*sqrt(energy)
10	format(a80)
c
	call read_fk(nk,fk)
c
c  initialization of k, x, y, and z arrays
	do 60 ix = 1, nx
	  x(ix) = xmin + (ix-1)*dx
60	continue
	do 70 iy = 1, ny
	  y(iy) = ymin + (iy-1)*dy
70	continue
	do 80 iz = 1, nz
	  z(iz) = zmin + (iz-1)*dz
80	continue
	do 90 ik = 1, nk
	  k(ik) = k0 + (ik-1)*dk
90	continue
c
c  main read-in routine
	open(unit = 20, file = FilexK)
	do 100 ik = 1, nk
	  do 101 itheta = 1, ntheta
	    do 102 iphi = 1, nphi
              read(20,*) theta(itheta,iphi), phi(itheta,iphi),
     &                   xK(ik,itheta,iphi)
	      theta(itheta,iphi) = theta(itheta,iphi)*degtorad
	      phi(itheta,iphi) = (phi(itheta,iphi) - rotate)*degtorad
102         continue
101	  continue
100	continue
	close(20)
c
c  initialization of angles and trigonometrics
        do 200 itheta = 1, ntheta
          do 200 iphi = 1, nphi
	    ctheta(itheta,iphi) = cos(theta(itheta,iphi))
	    stheta(itheta,iphi) = sin(theta(itheta,iphi))
	    kx(itheta,iphi) = ctheta(itheta,iphi)*cos(phi(itheta,iphi))
	    ky(itheta,iphi) = ctheta(itheta,iphi)*sin(phi(itheta,iphi))
	    kz(itheta,iphi) = stheta(itheta,iphi)
200	continue
c
c  loop over each position in U(r) 
	nxyz = 0
	do 300 ix = 1, nx
	  do 301 iy = 1, ny
	    do 302 iz = 1, nz
	    dUr = (0., 0.)
	    r = sqrt(x(ix)*x(ix) + y(iy)*y(iy) + z(iz)*z(iz))
c  integrate over hologram solid angle
	    do 303 itheta = 1, ntheta
	      do 304 iphi = 1, nphi
		xkx = x(ix)*kx(itheta,iphi)
		yky = y(iy)*ky(itheta,iphi)
	    	zkz = z(iz)*kz(itheta,iphi)
		costhetark = (xkx + yky + zkz)/r
c  phase-sum over different wavenumbers
		  do 305 ik = 1, nk
	          call interpolate_ff(ik,costhetark,fk,ff)
                    krkr = k(ik)*(xkx + yky + zkz) - k(ik)*r
                    kern = cmplx(cos(-krkr),sin(-krkr))
	dUr = dUr + xK(ik,itheta,iphi)*kern*ff/(100.*k(ik))
305	          continue
	Ur(ix,iy,iz) = Ur(ix,iy,iz) + ctheta(itheta,iphi)*exp(real(dUr))
c  end phase-sum
304	        continue
303	      continue
cccc	      write(30,30) Ur
	Ur(ix,iy,iz) = abs(Ur(ix,iy,iz))
	Ur_ave = Ur_ave + Ur(ix,iy,iz)
c  end hologram solid angle integration
	      nxyz = nxyz + 1
302	    continue
301	  continue
300     continue
c
        Ur_ave = Ur_ave/real(nxyz)
        open(unit = 30, file = FileUr)
        do 320 ix = 1, nx
	  do 320 iy = 1, ny
            do 320 iz = 1, nz
            r2 = x(ix)*x(ix) + y(iy)*y(iy) + z(iz)*z(iz)
            if (r2 .ne. 0.) then
cccc              write(30,30) (Ur_ave - Ur(ix,iy,iz)), 0.
              write(30,30) x(ix), y(iy), z(iz), (Ur_ave - Ur(ix,iy,iz)), 0.
            else
cccc              write(30,30) 0., 0.
              write(30,30) x(ix), y(iy), z(iz), 0., 0.
	    endif
320	continue	
cccc 
cccc30	format(2(2x,e12.5))
30      format(3(2x,f6.2),2(2x,e12.5))
c
	close(30)
	end
c
        subroutine read_fk(nk, fk)
c  this subroutine reads in the scattering factor magnitudes that are output
c  from fkSWIFT.f
c
        real fk(nk,0:180)
c
        open(unit = 40, file = 'fk.in')
        do 400 ik = 1, nk
          read(40,*) k_no_use
          do 400 itheta = 0, 180
            read(40,*) fk(ik,itheta)
400     continue
        close(40)
        return
        end
c
        subroutine interpolate_ff(ik, costhetark, fk, ff)
c  this subroutine interpolates the scattering factor magnitude for the angle
c  thetark, given cos(thetark)
c
        real costhetark, fk(11,0:180)
c
        thetark = aacos(costhetark)
        ithetark = int(thetark)
        delta = abs(thetark - ithetark)
c
        ff1 = fk(ik,ithetark)
        if (ithetark .eq. 180) then ithetark = 178
        ff2 = fk(ik,ithetark + 1)
c
        ff = ff1 + (ff2 - ff1)*delta
c
        return
        end
c
        function aacos(costhetark)
c  this function calculates acos(cos(thetark), but allows for a small amount
c  of round-off error when cos(thetark) is calculated as a dot product
c
        real costhetark, aacos
c
        if (abs(costhetark) .gt. 1.0001) stop 'Error 1 in aacos'
        if (abs(costhetark) .gt. 1) costhetark = sign(1., costhetark)
        aacos = acos(costhetark)
        return
        end

